import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/create_folder_result_entity.dart';
import 'package:vone_ipfs/generated/json/base/json_convert_content.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/utils/toast_utils.dart';

///
///创建文件夹
///
class CreateFolderView extends StatefulWidget {
  final String parentId;
  final Function callBack;

  const CreateFolderView(this.parentId, this.callBack, {Key? key})
      : super(key: key);

  @override
  CreateFolderViewState createState() => new CreateFolderViewState();
}

class CreateFolderViewState extends State<CreateFolderView> {
  late TextEditingController _controller;

  @override
  Widget build(BuildContext context) {
    return Material(
        type: MaterialType.transparency,
        child: Column(children: <Widget>[
          Expanded(
            child: GestureDetector(
              behavior: HitTestBehavior.translucent,
              onTap: () {
                Navigator.of(context).pop();
              },
              child: Container(),
            ),
          ),
          Container(
            height: MediaQuery.of(context).size.height * 0.9,
            decoration: const BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(10.0),
                  topRight: Radius.circular(10.0)),
            ),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                _buildTitle(),
                const SizedBox(
                  height: 20,
                ),
                Center(
                  child: Container(
                    padding: const EdgeInsets.all(30),
                    color: const Color(0xFFFFD23D).withOpacity(0.1),
                    child: IPFSUtils.getIconFont(
                      0xe622,
                      const Color(0xFFFFD23D),
                      60.0,
                    ),
                  ),
                ),
                Container(
                  margin: EdgeInsets.all(20),
                  decoration: BoxDecoration(
                      color: const Color(0xFFF4F4F4),
                      borderRadius: BorderRadius.circular(8)),
                  child: TextField(
                    decoration: InputDecoration(
                      border: InputBorder.none,
                      suffixIcon: InkWell(
                        child: const Icon(
                          Icons.cancel,
                          size: 20,
                          color: Colors.grey,
                        ),
                        onTap: () {
                          setState(() {
                            _controller.clear();
                          });
                        },
                      ),
                    ),
                    textAlign: TextAlign.center,
                    maxLines: 1,
                    controller: _controller,
                    onChanged: (value) {},
                  ),
                )
              ],
            ),
          )
        ]));
  }

  @override
  void initState() {
    super.initState();
    _controller = TextEditingController(text: '新建文件夹');
  }

  @override
  void dispose() {
    super.dispose();
  }

  Widget _buildTitle() {
    return Container(
      width: double.infinity,
      padding: const EdgeInsets.all(10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          InkWell(
            child: const Icon(
              Icons.close,
              color: Colors.black,
              size: 25,
            ),
            onTap: () {
              Navigator.of(context).pop();
            },
          ),
          const Text(
            "新建文件夹",
            style: TextStyle(color: Colors.black, fontSize: 18),
          ),
          InkWell(
            child: const Text(
              '完成',
              style: TextStyle(
                  color: Color(0xFF1684FC),
                  fontSize: 18,
                  fontWeight: FontWeight.bold),
            ),
            onTap: () {
              if (_controller.text.isEmpty) {
                ToastUtils.show('请输入文件夹名称');
                return;
              }
              _createFolder();
            },
          )
        ],
      ),
    );
  }

  _createFolder() async {
    Response response;
    Dio dio = Dio();
    Map<String, dynamic> param = {
      "parentID": int.parse(widget.parentId),
      "name": _controller.text,
      "userID": IPFSUtils.instance!.userId,
      "secretkey": IPFSUtils.instance!.secretKey,
    };
    response = await dio.post<String>(
        '${IPFSUtils.instance!.requestUrl}${HttpApi.createFolder}',
        data: param);
    print(response.data.toString());
    CreateFolderResultEntity result =
        JsonConvert.fromJsonAsT<CreateFolderResultEntity>(
            jsonDecode(response.data.toString()));
    if (response.statusCode == 200) {
      if (result.code == 0) {
        widget.callBack();
        Navigator.of(context).pop();
      } else {
        ToastUtils.show(result.msg??'文件夹创建失败');
      }
    } else {
      ToastUtils.show('文件夹创建失败');
    }
  }
}
