import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/component/multiple_image_view.dart';
import 'package:vone_ipfs/component/single_image_view.dart';
import 'package:vone_ipfs/component/vone_image.dart';

class ImageUploadView extends StatefulWidget {
  ///图片的宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///已有的图片列表
  final List<UploadResultEntity> images;

  ///图片形状
  final ImageShape shape;

  ///上传模式 默认多文件上传
  final UploadMode uploadMode;

  ///上传成功的回调
  final ValueChanged<List<UploadResultEntity>>? valueChangedCallback;

  ///添加图片的按钮
  final Widget? addImageWidget;

  /// 添加图片按钮的位置
  final UploadBtnPosition btnPosition;
  final int itemCount;
  final String dirId;
  final String dirName;
  final String placeholder;
  bool isFixedDir;

  ///是否可以编辑
  bool isCanEdit;
  IPFSFileType fileType;

  //最大图片数
  int maxCount;

  //是否允许云端
  bool isAllowCloud;

  ImageUploadView(
      {Key? key,
      this.imageWidth = 110,
      this.imageHeight = 110,
      required this.images,
      required this.dirId,
      required this.dirName,
      this.isFixedDir = false,
      this.uploadMode = UploadMode.MULTIPLE,
      this.fileType = IPFSFileType.image,
      this.valueChangedCallback,
      this.addImageWidget,
      this.isAllowCloud = true,
      this.itemCount = 3,
      this.isCanEdit = true,
      this.placeholder = '',
      this.maxCount = -1,
      this.btnPosition = UploadBtnPosition.RIGHT,
      this.shape = ImageShape.None})
      : super(key: key);

  @override
  ImageUploadViewState createState() => ImageUploadViewState();
}

class ImageUploadViewState extends State<ImageUploadView> {
  @override
  Widget build(BuildContext context) {
    if (widget.uploadMode == UploadMode.SINGLE) {
      return SingleImageView(
          dirId: widget.dirId,
          dirName: widget.dirName,
          isFixedDir: widget.isFixedDir,
          imageWidth: widget.imageWidth,
          imageHeight: widget.imageHeight,
          placeholder: widget.placeholder,
          shape: widget.shape,
          isAllowCloud:widget.isAllowCloud,
          isCanEdit: widget.isCanEdit,
          fileType: widget.fileType,
          addImageWidget: widget.addImageWidget,
          imageEntity: widget.images.isNotEmpty ? widget.images[0] : null,
          singleCallback: (value) {
            if (widget.valueChangedCallback != null) {
              widget.valueChangedCallback!([value]);
            }
          });
    } else {
      return MultipleImageView(
        images: widget.images,
        imageWidth: widget.imageWidth,
        imageHeight: widget.imageHeight,
        addImageWidget: widget.addImageWidget,
        placeholder: widget.placeholder,
        itemCount: widget.itemCount,
        btnPosition: widget.btnPosition,
        isCanEdit: widget.isCanEdit,
        isAllowCloud:widget.isAllowCloud,
        shape: widget.shape,
        fileType: widget.fileType,
        dirId: widget.dirId,
        dirName: widget.dirName,
        isFixedDir: widget.isFixedDir,
        maxCount: widget.maxCount,
        valueChangedCallback: widget.valueChangedCallback,
      );
    }
  }

  @override
  void initState() {
    super.initState();
  }
}
