import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/pages/file_preview_page.dart';
import 'package:vone_ipfs/pages/image_picker_dialog.dart';
import 'package:vone_ipfs/pages/online_file_page.dart';
import 'package:vone_ipfs/pages/upload_progress_dialog.dart';
import 'package:vone_ipfs/utils/ifps_dialog_utils.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/component/vone_image.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/utils/toast_utils.dart';

import 'defult_upload_view.dart';
import 'intrinsic_grid_view.dart';

///
/// 多图片上传
///
class MultipleImageView extends StatefulWidget {
  ///图片的宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///已有的图片列表
  List<UploadResultEntity> images;

  ///图片形状
  final ImageShape shape;

  ///上传成功的回调
  final ValueChanged<List<UploadResultEntity>>? valueChangedCallback;

  ///添加图片的按钮
  final Widget? addImageWidget;

  /// 添加图片按钮的位置
  final UploadBtnPosition btnPosition;
  final int itemCount;
  final int maxCount;
  final String dirId;
  final String dirName;
  final String placeholder;
  final bool isFixedDir;
  final bool isCanEdit;
  bool isAllowCloud;
  final IPFSFileType fileType;

  MultipleImageView(
      {Key? key,
      this.imageWidth = 110,
      this.imageHeight = 110,
      required this.images,
      this.valueChangedCallback,
      this.addImageWidget,
      this.itemCount = 3,
      this.maxCount = -1,
      this.placeholder = '',
      this.isCanEdit = true,
      this.isAllowCloud = true,
      this.fileType = IPFSFileType.image,
      required this.dirId,
      required this.dirName,
      required this.isFixedDir,
      this.btnPosition = UploadBtnPosition.RIGHT,
      this.shape = ImageShape.None})
      : super(key: key);

  @override
  MultipleImageViewState createState() => new MultipleImageViewState();
}

class MultipleImageViewState extends State<MultipleImageView> {
  List<UploadResultEntity> _imageList = [];
  int maxCount = -1;

  // bool _showProgress = false;
  // int _count = 0;
  // int _total = 0;
  @override
  Widget build(BuildContext context) {
    if (widget.images.isNotEmpty) {
      _imageList = widget.images;
      widget.images = [];
    }
    int size = _imageList.length;
    if (widget.isCanEdit) {
      size = size + 1;
    }
    return Container(
      margin: const EdgeInsets.only(left: 10),
      child: Column(
        children: [
          widget.btnPosition == UploadBtnPosition.TOP
              ? _buildTopBtn()
              : const Text(''),
          IntrinsicGridView.vertical(
            children: [for (int i = 0; i < size; i++) _buildItem(i)],
            columnCount: widget.itemCount,
            horizontalSpace: 5,
            verticalSpace: 5,
          )
        ],
      ),
    );
  }

  Widget _buildItem(int index) {
    if (_imageList.isEmpty || index == _imageList.length) {
      return widget.btnPosition == UploadBtnPosition.TOP
          ? const Text('')
          : InkWell(
              child: widget.addImageWidget ??
                  DefaultUploadView(
                    width: widget.imageWidth,
                    height: widget.imageHeight,
                  ),
              onTap: () {
                _showPickerDialog();
              },
            );
    } else {
      return Container(
        width: widget.imageWidth,
        height: widget.imageHeight,
        alignment: Alignment.topLeft,
        child: Stack(
          alignment: Alignment.topRight,
          children: [
            InkWell(
              child: Stack(
                alignment: Alignment.center,
                children: [
                  VoneImage(
                    imageWidth: double.infinity,
                    imageHeight: double.infinity,
                    imageUrl: _imageList[index].coverPath.isEmpty
                        ? _imageList[index].filePath
                        : _imageList[index].coverPath,
                    shape: widget.shape,
                    placeholder: widget.placeholder,
                    boxFit: BoxFit.fill,
                  ),
                  Visibility(
                    child: const Icon(
                      Icons.play_circle_filled,
                      size: 30,
                      color: Colors.white,
                    ),
                    visible: _imageList[index].coverPath.isNotEmpty,
                  )
                ],
              ),
              onTap: () {
                Navigator.push(context, MaterialPageRoute(builder: (context) {
                  return FilePreviewPage(fileUrl: _imageList[index].filePath);
                }));
              },
            ),
            Visibility(
              child: InkWell(
                child: Container(
                  width: 20,
                  height: 20,
                  decoration: BoxDecoration(
                      color: Colors.black.withOpacity(0.6),
                      borderRadius: BorderRadius.circular(14)),
                  child: const Icon(
                    Icons.close,
                    size: 20,
                    color: Colors.white,
                  ),
                ),
                onTap: () {
                  setState(() {
                    _imageList.removeAt(index);
                  });
                  if (widget.valueChangedCallback != null) {
                    widget.valueChangedCallback!(_imageList);
                  }
                },
              ),
              visible: widget.isCanEdit,
            )
          ],
        ),
      );
    }
  }

  @override
  void initState() {
    super.initState();
    maxCount = widget.maxCount;
  }

  _showPickerDialog() {
    if (maxCount > 0 && _imageList.length >= maxCount) {
      ToastUtils.show('已超过最大上传文件数');
      return;
    }
    if (widget.isAllowCloud) {
      showModalBottomSheet(
          backgroundColor: Colors.transparent,
          context: context, //state.context,
          builder: (BuildContext context) {
            return ImagePickerDialog(
              localCallback: () {
                _localUpload();
              },
              cloudCallback: () {
                Navigator.push(context, MaterialPageRoute(builder: (context) {
                  return OnlineFilePage(
                    widget.dirId,
                    widget.dirName,
                    (result) {
                      setState(() {
                        if (result.isNotEmpty) {
                          result.forEach((element) {
                            _imageList.add(UploadResultEntity(
                                IPFSUtils.instance!
                                    .getFilePath(element.id, element.fileName),
                                "",
                                element.fileName));
                          });
                          if (widget.valueChangedCallback != null) {
                            widget.valueChangedCallback!(_imageList);
                          }
                          setState(() {});
                        }
                      });
                    },
                    fileType: widget.fileType,
                    maxCount: maxCount > 0 ? maxCount - _imageList.length : 9,
                  );
                }));
              },
            );
          });
    } else {
      _localUpload();
    }
  }

  void _localUpload() {
    if (widget.isFixedDir) {
      _uploadImage();
    } else {
      IPFSDialog.showImageUpload(
          context,
          widget.imageWidth,
          widget.imageHeight,
          widget.dirId,
          widget.dirName,
          UploadMode.MULTIPLE,
          widget.fileType, (value) {
        _imageList.addAll(value);
        if (widget.valueChangedCallback != null) {
          widget.valueChangedCallback!(_imageList);
        }
        setState(() {});
      });
    }
  }

  // _showUploadDialog() {
  //   if (_showProgress) {
  //     return;
  //   }
  //   print("------ $_count");
  //   showGeneralDialog(
  //       context: context,
  //       barrierLabel: "",
  //       barrierColor: Colors.black.withOpacity(0.5),
  //       transitionDuration: const Duration(milliseconds: 500),
  //       barrierDismissible: true,
  //       pageBuilder: (BuildContext context, Animation animation,
  //           Animation secondaryAnimation) {
  //         return Scaffold(
  //           backgroundColor: Colors.transparent,
  //           body: Center(
  //             child: UploadProgressDialog(
  //               count: _count,
  //               total: _total,
  //             ),
  //           ),
  //         );
  //       });
  // }

  Widget _buildTopBtn() {
    return widget.isCanEdit
        ? InkWell(
            child: widget.addImageWidget ??
                Container(
                  alignment: Alignment.center,
                  width: double.infinity,
                  padding: const EdgeInsets.symmetric(vertical: 15),
                  margin: const EdgeInsets.only(top: 10, bottom: 10, right: 10),
                  decoration: BoxDecoration(
                      borderRadius: BorderRadius.circular(3),
                      color: Colors.blue),
                  child: const Text(
                    '选择图片',
                    style: TextStyle(color: Colors.white, fontSize: 15),
                  ),
                ),
            onTap: widget.isCanEdit
                ? () {
                    _showPickerDialog();
                  }
                : null,
          )
        : Text('');
  }

  void _uploadImage() {
    IPFSUtils.instance!.imageUpload(context, (value) {
      _imageList.add(value);
      if (widget.valueChangedCallback != null) {
        widget.valueChangedCallback!(_imageList);
      }
      setState(() {});
    }, widget.dirId, fileType: widget.fileType);
  }
}
