import 'dart:convert';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:flutter_easyrefresh/easy_refresh.dart';
import 'package:vone_ipfs/bean/folder_data_entity.dart';
import 'package:vone_ipfs/bean/select_folder_entity.dart';
import 'package:vone_ipfs/generated/json/base/json_convert_content.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/component/create_folder_view.dart';

class SelectDirView extends StatefulWidget {
  final ValueChanged<SelectFolderEntity> callback;
  String dirId;

  SelectDirView(
    this.dirId,
    this.callback, {
    Key? key,
  }) : super(key: key);

  @override
  SelectDirViewState createState() => new SelectDirViewState();
}

class SelectDirViewState extends State<SelectDirView> {
  List<FolderDataDataList> _list = [];
  String _parentId = "";
  String _selectId = '';
  String _selectName = '';

  ///当前页
  int _page = 1;

  ///每页条数
  final int _pageSize = 15;
  bool isMore = false;
  late EasyRefreshController _controller;

  _dismissDialog() {
    Navigator.of(context).pop();
  }

  @override
  Widget build(BuildContext context) {
    return Material(
      type: MaterialType.transparency,
      child: Column(
        children: <Widget>[
          Expanded(
            child: GestureDetector(
              behavior: HitTestBehavior.translucent,
              onTap: _dismissDialog,
              child: Container(),
            ),
          ),
          Container(
            decoration: const BoxDecoration(
              color: Colors.white,
              borderRadius: BorderRadius.only(
                  topLeft: Radius.circular(10.0),
                  topRight: Radius.circular(10.0)),
            ),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                _buildTitle(),
                const Divider(
                  color: Color(0xFFF1F1F6),
                  height: 1,
                ),
                Container(
                  height: MediaQuery.of(context).size.height * 0.75,
                  child: _list.isNotEmpty
                      ? EasyRefresh(
                          header: ClassicalHeader(
                              enableInfiniteRefresh: false,
                              refreshText: '下拉刷新',
                              refreshReadyText: '释放刷新',
                              refreshingText: '正在刷新...',
                              refreshedText: '刷新完成',
                              refreshFailedText: '刷新失败',
                              noMoreText: '没有更多数据',
                              infoText: '更新于 %T',
                              infoColor: const Color(0xFF999999)),
                          footer: ClassicalFooter(
                              enableInfiniteLoad: true,
                              loadText: '拉动加载',
                              loadReadyText: '释放加载',
                              loadingText: '正在加载...',
                              loadedText: '加载完成',
                              loadFailedText: '加载失败',
                              noMoreText: '没有更多数据',
                              infoText: '更新于 %T',
                              infoColor: const Color(0xFF999999)),
                          controller: _controller,
                          onRefresh: () async {
                            setState(() {
                              _page = 1;
                            });
                            _getFolderData(_parentId, true);
                            _controller.finishRefresh();
                            _controller.resetLoadState();
                          },
                          onLoad: isMore
                              ? () async {
                                  _controller.finishLoad(noMore: !isMore);
                                  _getFolderData(_parentId, true);
                                }
                              : null,
                          child: ListView(
                            children: _buildDir(),
                            shrinkWrap: true,
                          ))
                      : _buildNullFolder(),
                ),
                Container(
                  padding: EdgeInsets.all(20),
                  child: Row(
                    mainAxisAlignment: MainAxisAlignment.spaceBetween,
                    children: [
                      InkWell(
                        child: Row(
                          children: const [
                            Icon(
                              Icons.create_new_folder,
                              color: Color(0xFF1684FC),
                              size: 30,
                            ),
                            SizedBox(
                              width: 10,
                            ),
                            Text(
                              '新建文件夹',
                              style: TextStyle(
                                  color: Color(0xFF1684FC), fontSize: 16),
                            )
                          ],
                        ),
                        onTap: () {
                          _showCreateFolerDialog();
                        },
                      ),
                      InkWell(
                        child: Container(
                          height: 40,
                          padding: const EdgeInsets.symmetric(horizontal: 15),
                          alignment: Alignment.center,
                          decoration: BoxDecoration(
                              color: const Color(0xFF1684FC),
                              borderRadius: BorderRadius.circular(40)),
                          child: const Text(
                            '上传到此',
                            style: TextStyle(color: Colors.white, fontSize: 16),
                          ),
                        ),
                        onTap: () {
                          print(_selectId);
                          print(_selectName);
                          widget.callback(
                              SelectFolderEntity(_selectId, _selectName));
                          Navigator.of(context).pop();
                        },
                      )
                    ],
                  ),
                )
                // ListView.builder(
                //   itemBuilder: (context, index) {
                //     return _buildDirItem(index);
                //   },
                //   shrinkWrap: true,
                // ),
              ],
            ),
          ),
        ],
      ),
    );
  }

  Widget _buildTitle() {
    return Container(
      width: double.infinity,
      padding: const EdgeInsets.all(10),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceBetween,
        children: [
          Visibility(
            child: InkWell(
              child: const Text(
                '返回',
                style: TextStyle(fontSize: 16, color: Color(0xFF1684FC)),
              ),
              onTap: () {
                _page = 1;
                _getFolderData(_parentId, true);
              },
            ),
          ),
          const Text(
            "选择文件夹",
            style: TextStyle(color: Colors.black, fontSize: 18),
          ),
          InkWell(
            child: const Icon(
              Icons.close,
              color: Colors.black,
              size: 25,
            ),
            onTap: () {
              Navigator.of(context).pop();
            },
          )
        ],
      ),
    );
  }

  Widget _buildNullFolder() {
    return Center(
      child: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          IPFSUtils.getIconFont(
            0xe622,
            const Color(0xFFFFD23D),
            60.0,
          ),
          const SizedBox(
            height: 10,
          ),
          Text(
            '这里是空的',
            style:
                TextStyle(fontSize: 14, color: Colors.black.withOpacity(0.6)),
          )
        ],
      ),
    );
  }

  List<Widget> _buildDir() {
    List<Widget> list = [];
    for (int i = 0; i < _list.length; i++) {
      list.add(_buildDirItem(i));
    }
    return list;
  }

  Widget _buildDirItem(int index) {
    String time = _list[index].updated.toString() + "000";
    String dataeTime = DateTime.fromMillisecondsSinceEpoch(int.parse(time))
        .toLocal()
        .toString()
        .substring(0, 19);

    return InkWell(
      child: Container(
        margin: const EdgeInsets.only(left: 20, top: 20),
        child: Row(
          children: [
            const Icon(
              IconData(0xe622,
                  fontFamily: 'iconfont', fontPackage: 'vone_ipfs'),
              color: Color(0xFFFFD23D),
              size: 25,
            ),
            const SizedBox(
              width: 20,
            ),
            Expanded(
                child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text(
                  _list[index].fileName ?? '未知',
                  style: const TextStyle(fontSize: 16, color: Colors.black87),
                ),
                Text(
                  dataeTime,
                  style: const TextStyle(fontSize: 14, color: Colors.grey),
                ),
              ],
            ))
          ],
        ),
      ),
      onTap: () {
        setState(() {
          _selectName = _list[index].fileName!;
          _selectId = _list[index].id.toString();
        });
        _page = 1;
        _getFolderData(_selectId, false);
      },
    );
  }

  @override
  void initState() {
    super.initState();
    _controller = EasyRefreshController();
    _parentId = widget.dirId;
    _getFolderData(widget.dirId, true);
  }

  @override
  void dispose() {
    super.dispose();
  }

  ///显示创建文件夹的dialog
  _showCreateFolerDialog() {
    showModalBottomSheet(
        context: context,
        backgroundColor: Colors.transparent,
        isScrollControlled: true,
        builder: (BuildContext context) {
          return CreateFolderView(_selectId, () {
            _page = 1;
            _getFolderData(_selectId, false);
          });
        });
  }

  ///获取文件夹列表
  _getFolderData(parentId, isBack) async {
    print("parentId  $parentId");
    Response response;
    Dio dio = Dio();
    Map<String, dynamic> param = {
      "userID": IPFSUtils.instance!.userId,
      "onlyDir": 'true',
      "secretKey": IPFSUtils.instance!.secretKey,
      "fileID": parentId,
      "page": _page,
      "pageSize": _pageSize
    };
    response =
        await dio.get<String>('${IPFSUtils.instance!.requestUrl}${HttpApi.getFolderList}', queryParameters: param);
    print(response.data.toString());
    if (200 == response.statusCode) {
      FolderDataEntity fileData = JsonConvert.fromJsonAsT<FolderDataEntity>(
          jsonDecode(response.data.toString()));
      if (fileData.code == 0) {
        if (fileData.data != null) {
          _parentId = fileData.data!.parent!.parentID.toString();
          if (_parentId.isEmpty || _parentId == 'null') {
            _parentId = widget.dirId;
          }
          if (isBack && widget.dirId == widget.dirId) {
            _parentId = widget.dirId;
          }
          setState(() {});
        } else {
          _parentId = widget.dirId;
        }
        if (fileData.data!.xList != null && fileData.data!.xList!.isNotEmpty) {
          setState(() {
            _selectId = parentId;
          });
          _setListData(fileData.data!.xList!);
        } else {
          setState(() {
            _list = [];
            _selectId = parentId;
          });
        }
      } else {
        setState(() {
          _list = [];
        });
      }
    } else {
      setState(() {
        _list = [];
      });
    }
  }

  _setListData(List<FolderDataDataList> data) {
    if (_page == 1) {
      _list = [];
    }
    if (data.isNotEmpty) {
      if (data.length < _pageSize) {
        isMore = false;
      } else {
        isMore = true;
      }
      _list.addAll(data);
      _page += 1;
    } else {
      isMore = false;
    }
    setState(() {});
  }
}
