import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/component/image_upload_view.dart';
import 'package:vone_ipfs/pages/file_preview_page.dart';
import 'package:vone_ipfs/pages/image_picker_dialog.dart';
import 'package:vone_ipfs/pages/online_file_page.dart';
import 'package:vone_ipfs/utils/ifps_dialog_utils.dart';
import 'package:vone_ipfs/utils/ipfs_utils.dart';
import 'package:vone_ipfs/component/vone_image.dart';

import 'defult_upload_view.dart';

///
/// 单图片上传
///
class SingleImageView extends StatefulWidget {
  ///图片的宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///图片形状
  final ImageShape shape;
  final String placeholder;
  String dirId;
  String dirName;
  bool isFixedDir;
  bool isCanEdit;

  //是否允许云端
  bool isAllowCloud;

  ///添加图片的按钮
  final Widget? addImageWidget;
  final UploadResultEntity? imageEntity;
  ValueChanged<UploadResultEntity>? singleCallback;
  IPFSFileType fileType;

  SingleImageView(
      {Key? key,
      this.imageWidth = 110,
      this.imageHeight = 110,
      required this.dirId,
      required this.dirName,
      required this.isFixedDir,
      this.shape = ImageShape.None,
      this.fileType = IPFSFileType.image,
      this.addImageWidget,
      this.singleCallback,
      this.isAllowCloud = true,
      this.isCanEdit = true,
      this.placeholder = '',
      this.imageEntity})
      : super(key: key);

  @override
  SingleImageViewState createState() => new SingleImageViewState();
}

class SingleImageViewState extends State<SingleImageView> {
  UploadResultEntity? _image;

  @override
  Widget build(BuildContext context) {
    if (widget.imageEntity != null && _image == null) {
      _image = widget.imageEntity;
      widget.imageEntity == null;
    }
    return InkWell(
      child: Container(
        width: widget.imageWidth,
        height: widget.imageHeight,
        child: _image == null
            ? (widget.addImageWidget ??
                DefaultUploadView(
                  width: widget.imageWidth,
                  height: widget.imageHeight,
                ))
            : Stack(
          alignment: Alignment.center,
                children: [
                  VoneImage(
                    imageWidth: double.infinity,
                    imageHeight: double.infinity,
                    imageUrl: _image!.coverPath.isEmpty
                        ? _image!.filePath
                        : _image!.coverPath,
                    shape: widget.shape,
                    placeholder: widget.placeholder,
                    boxFit: BoxFit.fill,
                  ),
                  Visibility(
                    child: const Icon(
                      Icons.play_circle_filled,
                      size: 30,
                      color: Colors.white,
                    ),
                    visible: _image!.coverPath.isNotEmpty,
                  )
                ],
              ),
      ),
      onTap: widget.isCanEdit
          ? () {
              _showPickerDialog();
            }
          : () {
              Navigator.push(context, MaterialPageRoute(builder: (context) {
                return FilePreviewPage(
                  fileUrl: _image!.coverPath.isEmpty
                      ? _image!.filePath
                      : _image!.coverPath,
                );
              }));
            },
    );
  }

  @override
  void initState() {
    super.initState();
  }

  @override
  void dispose() {
    super.dispose();
  }

  _showPickerDialog() {
    if (widget.isAllowCloud) {
      showModalBottomSheet(
          backgroundColor: Colors.transparent,
          context: context, //state.context,
          builder: (BuildContext context) {
            return ImagePickerDialog(
              localCallback: () {
                _localUpload();
              },
              cloudCallback: () async {
                Navigator.push(context, MaterialPageRoute(builder: (context) {
                  return OnlineFilePage(
                    widget.dirId,
                    widget.dirName,
                    (result) {
                      setState(() {
                        if (result.isNotEmpty) {
                          _image = UploadResultEntity(
                              IPFSUtils.instance!.getFilePath(
                                  result[0].id, result[0].fileName),
                              "",
                              result[0].fileName);
                          if (widget.singleCallback != null && _image != null) {
                            widget.singleCallback!(_image!);
                          }
                        }
                      });
                    },
                    maxCount: 1,
                    fileType: widget.fileType,
                  );
                }));
              },
            );
          });
    } else {
      _localUpload();
    }
  }

  void _localUpload() {
    if (widget.isFixedDir) {
      _uploadImage();
    } else {
      IPFSDialog.showImageUpload(
          context,
          widget.imageWidth,
          widget.imageHeight,
          widget.dirId,
          widget.dirName,
          UploadMode.SINGLE,
          widget.fileType, (value) {
        setState(() {
          _image = value[0];
          if (widget.singleCallback != null && _image != null) {
            widget.singleCallback!(_image!);
          }
        });
      });
    }
  }

  void _uploadImage() {
    print(widget.fileType);
    IPFSUtils.instance!.imageUpload(context, (value) {
      _image = value;
      if (widget.singleCallback != null && _image != null) {
        widget.singleCallback!(_image!);
      }
      setState(() {});
    }, widget.dirId, fileType: widget.fileType);
  }
}
