import 'dart:typed_data';

import 'package:flutter/material.dart';

class VoneImage extends StatefulWidget {
  ///图片宽度
  final double imageWidth;

  ///图片高度
  final double imageHeight;

  ///默认图片
  String? placeholder;

  ///图片地址
  final String imageUrl;

  ///图片形状
  ImageShape shape;

  ///圆角大小
  double radiusSize;

  final BoxFit boxFit;

  VoneImage(
      {Key? key,
      required this.imageWidth,
      required this.imageHeight,
      this.placeholder,
      required this.imageUrl,
      this.shape = ImageShape.None,
      this.boxFit = BoxFit.cover,
      this.radiusSize = 0})
      : super(key: key);

  @override
  VoneImageState createState() => VoneImageState();
}

class VoneImageState extends State<VoneImage> {
  final Uint8List kTransparentImage = Uint8List.fromList(<int>[
    0x89,
    0x50,
    0x4E,
    0x47,
    0x0D,
    0x0A,
    0x1A,
    0x0A,
    0x00,
    0x00,
    0x00,
    0x0D,
    0x49,
    0x48,
    0x44,
    0x52,
    0x00,
    0x00,
    0x00,
    0x01,
    0x00,
    0x00,
    0x00,
    0x01,
    0x08,
    0x06,
    0x00,
    0x00,
    0x00,
    0x1F,
    0x15,
    0xC4,
    0x89,
    0x00,
    0x00,
    0x00,
    0x0A,
    0x49,
    0x44,
    0x41,
    0x54,
    0x78,
    0x9C,
    0x63,
    0x00,
    0x01,
    0x00,
    0x00,
    0x05,
    0x00,
    0x01,
    0x0D,
    0x0A,
    0x2D,
    0xB4,
    0x00,
    0x00,
    0x00,
    0x00,
    0x49,
    0x45,
    0x4E,
    0x44,
    0xAE,
  ]);

  String packageName = "vone_ipfs";

  @override
  Widget build(BuildContext context) {
    if (widget.placeholder == null || widget.placeholder!.isEmpty) {
      widget.placeholder = 'assets/images/ic_image_loading.png';
    }

    print("------${packageName}");
    Widget child;

    final isNetwork = widget.imageUrl.startsWith(RegExp(r'^http'));
    if (isNetwork) {
      if (widget.imageUrl.endsWith("null") || widget.imageUrl.endsWith("/")) {
        child = Image.asset(
          widget.placeholder!,
          width: widget.imageWidth,
          height: widget.imageHeight,
          fit: widget.boxFit,
          package: packageName.isEmpty?null:packageName,
        );
      } else {
        child = Stack(
          children: [
            SizedBox(
              width: widget.imageWidth,
              height: widget.imageHeight,
              child: const Center(child: CircularProgressIndicator()),
            ),
            FadeInImage.memoryNetwork(
                placeholder: kTransparentImage,
                image: widget.imageUrl,
                width: widget.imageWidth,
                height: widget.imageHeight,
                fit: widget.boxFit,
                imageErrorBuilder: (context, error, _) {
                  return Image.asset(
                    widget.placeholder!,
                    width: widget.imageWidth,
                    height: widget.imageHeight,
                    package: packageName.isEmpty?null:packageName,
                    fit: widget.boxFit,
                  );
                })
          ],
        );
      }
    } else {
      if (widget.imageUrl.isEmpty || widget.imageUrl == 'null') {
        child = Image.asset(
          widget.placeholder!,
          fit: widget.boxFit,
          width: widget.imageWidth,
          height: widget.imageHeight,
          package: packageName.isEmpty?null:packageName,
        );
      } else {
        child = Image.asset(
          widget.imageUrl,
          fit: widget.boxFit,
          width: widget.imageWidth,
          height: widget.imageHeight,
          package: packageName.isEmpty?null:packageName,
        );
      }
    }
    if (widget.shape == ImageShape.FILL) {
      return ClipRRect(
        borderRadius: BorderRadius.circular(widget.radiusSize), // 圆角半径
        child: Container(
          width: widget.imageWidth,
          height: widget.imageHeight,
          child: child,
        ),
      );
    } else if (widget.shape == ImageShape.ROUND) {
      return ClipOval(
        child: Container(
          width: widget.imageWidth,
          height: widget.imageHeight,
          child: child,
        ),
      );
    } else {
      return child;
    }
  }

  @override
  void initState() {
    super.initState();
    _initPackageInfo();
  }

  @override
  void dispose() {
    super.dispose();
  }

  Future<void> _initPackageInfo() async {
    String pkName = 'vone_ipfs';
    if (widget.placeholder != null && widget.placeholder!.isNotEmpty) {
      pkName = '';
    }
    setState(() {
      packageName = pkName;
    });
  }
}

enum ImageShape {
  ///原图
  None,

  ///圆角
  FILL,

  ///圆形
  ROUND
}
