import 'dart:convert';
import 'dart:core';
import 'dart:io';
import 'package:dio/dio.dart';
import 'package:file_picker/file_picker.dart';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:vone_ipfs/bean/upload_file_entity.dart';
import 'package:vone_ipfs/bean/upload_result_entity.dart';
import 'package:vone_ipfs/generated/json/base/json_convert_content.dart';
import 'package:vone_ipfs/net/http_api.dart';
import 'package:vone_ipfs/utils/loading_utils.dart';
import 'package:vone_ipfs/utils/toast_utils.dart';
import 'package:wechat_assets_picker/wechat_assets_picker.dart';

// String USER_ID = "";
// String SECRET_KEY = "";
///文件类型
enum IPFSFileType {
  ///任务文件
  any,

  ///媒体
  media,
  image,
  video,
  audio,
  imageAndVideo,
  custom,
}

class IPFSUtils {
  String userId = "";
  String secretKey = "";
  String requestUrl = "";
  String downloadUrl = "";
  String previewUrl = "";

  static IPFSUtils? instance;

  IPFSUtils(this.secretKey, this.requestUrl, this.downloadUrl, this.previewUrl);

  static IPFSUtils getInstance(
    String secretKey, {
    String requestUrl = "http://hn.vonechain.com:88",
    String downloadUrl = "http://hn.vonechain.com:88",
    String previewUrl =
        "http://hn.vonechain.com:88/ipfsPreview/onlinePreview",
  }) {
    instance ??= IPFSUtils(secretKey, requestUrl, downloadUrl, previewUrl);
    return instance!;
  }

  void initUserId(String userId) {
    this.userId = userId;
  }

  ///图片音视频上传
  Future<void> imageUpload(
      context, ValueChanged<UploadResultEntity> changed, String parentId,
      {IPFSFileType fileType = IPFSFileType.image,
      CommonSortPathDelegate? sortPathDelegate,
      Function? progressCallback}) async {
    print(fileType);
    RequestType requestType = RequestType.image;
    if (fileType == IPFSFileType.image) {
      requestType = RequestType.image;
    } else if (fileType == IPFSFileType.audio) {
      requestType = RequestType.audio;
    } else if (fileType == IPFSFileType.video) {
      requestType = RequestType.video;
    } else if (fileType == IPFSFileType.media) {
      requestType = RequestType.all;
    } else if (fileType == IPFSFileType.imageAndVideo) {
      requestType = RequestType.common;
    }
    List<AssetEntity>? image = await AssetPicker.pickAssets(context,
        maxAssets: 1,
        requestType: requestType,
        sortPathDelegate: sortPathDelegate);
    if (image != null && image.length > 0) {
      File? file = await image[0].loadFile();
      if (file != null) {
        String filePath = file.path;
        if (filePath.isEmpty) {
          ToastUtils.show('选择的文件不存在,请重新选择');
          return;
        }
        voneFileUpload(context, filePath, changed, parentId,
            progressCallback: progressCallback);
      } else {
        ToastUtils.show('选择的文件不存在,请重新选择');
      }
    }
  }

  ///文件上传
  Future<void> fileUpload(context, ValueChanged<UploadResultEntity> changed,
      String parentId, IPFSFileType fileType,
      {Function? progressCallback, List<String>? allowedExtensions}) async {
    FilePickerResult? result;
    print(allowedExtensions);
    switch (fileType) {
      case IPFSFileType.any:
        result = await FilePicker.platform.pickFiles(type: FileType.any);
        break;
      case IPFSFileType.video:
        result = await FilePicker.platform.pickFiles(type: FileType.video);
        break;
      case IPFSFileType.image:
        result = await FilePicker.platform.pickFiles(type: FileType.image);
        break;
      case IPFSFileType.audio:
        result = await FilePicker.platform.pickFiles(type: FileType.audio);
        break;
      case IPFSFileType.media:
      case IPFSFileType.imageAndVideo:
        result = await FilePicker.platform.pickFiles(type: FileType.media);
        break;
      case IPFSFileType.custom:
        result = await FilePicker.platform.pickFiles(
            type: FileType.custom, allowedExtensions: allowedExtensions);
        break;
    }

    if (result != null &&
        result.files.first != null &&
        result.files.first.extension != null &&
        result.files.first.extension!.isNotEmpty) {
      PlatformFile file = result.files.first;
      if (file != null) {
        String filePath = file.path ?? '';
        if (filePath.isEmpty) {
          ToastUtils.show('选择的文件不存在,请重新选择');
          return;
        }
        // int fileSize = await file.length();
        // if (fileSize > 40 * 1024 * 1024) {
        //   ToastUtils.show('文件大小超出限制');
        //   return;
        // }

        voneFileUpload(context, filePath, changed, parentId,
            progressCallback: progressCallback);
      } else {
        ToastUtils.show('选择的文件不存在,请重新选择');
      }
    }
  }

  Future<void> voneFileUpload(BuildContext context, String filePath,
      ValueChanged<UploadResultEntity> changed, String parentId,
      {Function? progressCallback}) async {
    try {
      LoadingUtils().showLoadingDialog(context);
      String fileName = filePath.split("/").last;
      Map<String, dynamic> fileMap = {};
      fileMap['file'] =
          await MultipartFile.fromFile(filePath, filename: fileName);
      FormData formData = FormData.fromMap(fileMap);
      Response response;
      Dio dio = Dio();
      dio.options.headers = {
        'userID': userId,
        'parentID': parentId,
        'secretKey': secretKey
      };
      String url = "${IPFSUtils.instance!.requestUrl}${HttpApi.fileUpload}";
      print("uploadUrl---${url}");
      response = await dio.post<String>(url, data: formData,
          onSendProgress: (count, total) {
        if (progressCallback != null) {
          progressCallback(count, total);
        }
      });
      UploadFileEntity fileData = JsonConvert.fromJsonAsT<UploadFileEntity>(
          jsonDecode(response.data.toString()));
      print(response.data.toString());
      if (0 == fileData.code) {
        if (fileData.data != null) {
          String filePath =
              getFilePath(fileData.data!.id!, fileData.data!.fileName!);
          String coverPath = getCoverPath(fileData.data!.url!);
          coverPath = coverPath.isEmpty ? '' : Uri.decodeComponent(coverPath);
          print('coverPath$coverPath');
          print('filePath$filePath');
          UploadResultEntity entity =
              UploadResultEntity(filePath, coverPath, fileData.data!.fileName!);
          changed(entity);
        }
      } else {
        ToastUtils.show('文件上传失败');
      }
      LoadingUtils().dismiss(context);
    } catch (e) {
      print(e);
      LoadingUtils().dismiss(context);
      ToastUtils.show('文件上传失败');
    }
  }

  // static void imageUpload(BuildContext context, List<String> list, String dirId,
  //     {double width = 110, double height = 110}) {
  //   Navigator.push(
  //       context,
  //       MaterialPageRoute(
  //           builder: (context) => ImageUploadPage(
  //                 images: list,
  //                 dirId: dirId,
  //               )));
  // }

  static bool isDir(String type) {
    return type == 'folder';
  }

  ///根据文件的后缀名获取文件的图标
  static Icon getFileIconBySuffix(String fileName, {String? fileType}) {
    var type = fileName.split('.').last.toLowerCase();
    int iconName = 0xe858;
    Color color = const Color(0xFFFFD23D);
    if ('folder' == fileType) {
      iconName = 0xe622;
      color = const Color(0xFFFFD23D);
    } else {
      if (type == 'doc' || type == 'docx') {
        iconName = 0xe885;
        color = const Color(0xFF4D97FF);
      } else if (type == 'xls' || type == 'xlsx' || type == 'csv') {
        iconName = 0xedde;
        color = const Color(0xFF63C421);
      } else if (type == 'ppt' || type == 'pptx') {
        iconName = 0xe859;
        color = const Color(0xFFFF9743);
      } else if (type == 'cd' ||
          type == 'wave' ||
          type == 'aiff' ||
          type == 'wma' ||
          type == 'mpeg' ||
          type == 'mp3' ||
          type == 'wma' ||
          type == 'au' ||
          type == 'midi' ||
          type == 'realAudio' ||
          type == 'ape' ||
          type == 'aac') {
        iconName = 0xe6ed;
        color = const Color(0xFF8183F1);
      } else if (type == 'wmv' ||
          type == 'asf' ||
          type == 'asx' ||
          type == 'rm' ||
          type == 'rmvb' ||
          type == 'mp4' ||
          type == 'mov' ||
          type == 'm4v' ||
          type == 'avi' ||
          type == 'dat' ||
          type == 'mkv' ||
          type == 'flv' ||
          type == 'vob') {
        iconName = 0xe665;
        color = const Color(0xFF8183F1);
      } else if (type == 'pdf') {
        iconName = 0xe85b;
        color = const Color(0xFFFF5B5A);
      } else if (type == 'zip' ||
          type == 'rar' ||
          type == 'arj' ||
          type == 'jar' ||
          type == 'iso' ||
          type == 'uue' ||
          type == 'gz' ||
          type == 'tar' ||
          type == 'ace' ||
          type == 'cab' ||
          type == '7z' ||
          type == 'z') {
        iconName = 0xe85a;
        color = const Color(0xFF54C7F7);
      } else if (type == 'png' ||
          type == 'jpg' ||
          type == 'jpeg' ||
          type == 'bmp' ||
          type == 'heic' ||
          type == 'gif') {
        iconName = 0xe62e;
        color = const Color(0xFFFF7743);
      } else if (type == 'txt') {
        iconName = 0xe858;
        ;
        color = const Color(0xFF4D97FF);
      } else {
        iconName = 0xe858;
        ;
        color = Colors.grey;
      }
    }
    return getIconFont(iconName, color, 33);
  }

  //是否为允许选择的文件类型
  static bool isAllowFileType(String fileName, IPFSFileType fileType,
      {List<String>? allowedExtensions = const []}) {
    var type = fileName.split('.').last.toLowerCase();
    print('-------------$type');
    if (fileType == IPFSFileType.any) {
      return true;
    } else if (fileType == IPFSFileType.custom) {
      return allowedExtensions!.contains(type);
    } else {
      if ((type == 'cd' ||
              type == 'wave' ||
              type == 'aiff' ||
              type == 'wma' ||
              type == 'mpeg' ||
              type == 'mp3' ||
              type == 'wma' ||
              type == 'au' ||
              type == 'midi' ||
              type == 'realAudio' ||
              type == 'ape' ||
              type == 'aac') &&
          (fileType == IPFSFileType.audio)) {
        return true;
      } else if ((type == 'wmv' ||
              type == 'asf' ||
              type == 'asx' ||
              type == 'rm' ||
              type == 'rmvb' ||
              type == 'mp4' ||
              type == 'mov' ||
              type == 'm4v' ||
              type == 'avi' ||
              type == 'dat' ||
              type == 'mkv' ||
              type == 'flv' ||
              type == 'vob') &&
          (fileType == IPFSFileType.video ||
              fileType == IPFSFileType.media ||
              fileType == IPFSFileType.imageAndVideo)) {
        return true;
      } else if ((type == 'png' ||
              type == 'jpg' ||
              type == 'jpeg' ||
              type == 'bmp' ||
              type == 'heic' ||
              type == 'gif') &&
          (fileType == IPFSFileType.image ||
              fileType == IPFSFileType.media ||
              fileType == IPFSFileType.imageAndVideo)) {
        return true;
      }
    }
    return false;
  }

  ///加载iconfont的图标
  static Icon getIconFont(int iconName, color, double size) {
    return Icon(
      IconData(iconName, fontFamily: 'iconfont', fontPackage: 'vone_ipfs'),
      color: color,
      size: size,
    );
  }

  String getFilePath(fileId, fileName) {
    return '${IPFSUtils.instance!.downloadUrl}${HttpApi.fileDownload}?secretKey=$secretKey&fileID=$fileId&userID=$userId&fullfilename=$fileName';
  }

  String getCoverPath(String url) {
    if (url.isEmpty) {
      return "";
    }
    return '${IPFSUtils.instance!.downloadUrl}${HttpApi.coverDownload}?$url';
  }
}

// class CustomSortPathDelegate extends CommonSortPathDelegate {
//   const CustomSortPathDelegate();
//
//   @override
//   void sort(List<AssetPathEntity> list) {
//     ///...///
//
//     // In here you can check every path entities if you want.
//     // The only property we recommend to change is [name],
//     // And we have no responsibility for issues caused by
//     // other properties update.
//     for (final AssetPathEntity entity in list) {
//       // If the entity `isAll`, that's the "Recent" entity you want.
//       if (entity.isAll) {
//         entity.name = '全部';
//       } else if (entity.name == 'Camera') {
//         entity.name = '相机';
//       } else if (entity.name == 'Pictures') {
//         entity.name = '图片';
//       }
//     }
//   }
// }

///多文件还是单文件
enum UploadMode {
  ///单文件
  SINGLE,

  ///多文件
  MULTIPLE
}

///上传图片的按钮位置
enum UploadBtnPosition {
  ///图片上面
  TOP,

  ///图片右边
  RIGHT
}
